% svarest6.m: Estimates via maximum likelihood a six-variable VAR for
% prices, output, interest rates, Divisia money, the user cost of money,
% and commodity prices. Computes parameters, impulse responses, variance
% decompositions, and their standard errors.
%
% Options:
%
%       model 1 = Cholesky identification scheme
%       model 2 = Interest rate-money rule
%
%       q = lag length
%
%       level = 1 for data in log levels
%       level = 0 for data in growth rates
%
%       sample = 1 for 1967:1 - 1983:4 subsample
%       sample = 2 for 1984:1 - 1999:4 subsample
%       sample = 3 for 2000:1 - 2013:4 subsample
%
% Program written for MATLAB 8.3 (R2014a) by Peter Ireland, Department
% of Economics, Boston College (peter.ireland@bc.edu).
%
% =========================================================================
% =========================================================================

% =========================================================================
% Define globals, choose model and lag length, load data, select levels or
% growth rates, and choose sample period.
% =========================================================================

clear;

global model level Omegahat T q

model = 2;
q = 2;

load pyr.dat
load divm.dat
load divu.dat
load cp.dat

level = 1;
sample = 1;

% =========================================================================
% Set up data based on choices made above.
% =========================================================================

if level == 1

    svardatq(:,1:3) = pyr;

    svardatq(:,4) = divm(:,1);
    svardatq(:,5) = divu;

    svardatq(:,6) = cp;
    
    if sample == 1
        
        svardatq = svardatq(1:68,:);
        
    elseif sample == 2
        
        svardatq = svardatq(69:132,:);
        
    else
        
        svardatq = svardatq(133:188,:);
        
    end
    
else

    svardatq(:,1:2) = pyr(2:188,1:2) - pyr(1:187,1:2);
    svardatq(:,3) = pyr(2:188,3);

    svardatq(:,4) = divm(2:188,1) - divm(1:187,1);
    svardatq(:,5) = divu(2:188);

    svardatq(:,6) = cp(2:188) - cp(1:187);

    if sample == 1

        svardatq = svardatq(1:67,:);
        
    elseif sample == 2
        
        svardatq = svardatq(68:131,:);
        
    else
        
        svardatq = svardatq(132:187,:);
        
    end
    
end

% =========================================================================
% Estimate intercept and slope coefficients via equation-by-equation OLS.
% =========================================================================

T = length(svardatq(:,1)) - q;

Pt = svardatq(q+1:T+q,1);
Yt = svardatq(q+1:T+q,2);
Rt = svardatq(q+1:T+q,3);
Mt = svardatq(q+1:T+q,4);
Ut = svardatq(q+1:T+q,5);
Pcomt = svardatq(q+1:T+q,6);

Xt = zeros(T,6*q+1);

Xt(:,1) = ones(T,1);

for k = 1:q
    
    Xt(:,6*(k-1)+2:6*(k-1)+7) = svardatq(q+1-k:T+q-k,:);
    
end

XpX = Xt'*Xt;

Gammap = XpX\(Xt'*Pt);
Gammay = XpX\(Xt'*Yt);
Gammar = XpX\(Xt'*Rt);
Gammam = XpX\(Xt'*Mt);
Gammau = XpX\(Xt'*Ut);
Gammapcom = XpX\(Xt'*Pcomt);

Gamma = [ Gammap(2:6*q+1)' ; ...
          Gammay(2:6*q+1)' ; ...
          Gammar(2:6*q+1)' ; ...
          Gammam(2:6*q+1)' ; ...
          Gammau(2:6*q+1)' ; ...
          Gammapcom(2:6*q+1)' ; ...
          eye(6*(q-1)) zeros(6*(q-1),6) ];
        
nu = [ Gammap(1) ; ...
       Gammay(1) ; ...
       Gammar(1) ; ...
       Gammam(1) ; ...
       Gammau(1) ; ...
       Gammapcom(1) ; ...
       zeros(6,1) ];

pt = Pt - Xt*Gammap;
yt = Yt - Xt*Gammay;
rt = Rt - Xt*Gammar;
mt = Mt - Xt*Gammam;
ut = Ut - Xt*Gammau;
pcomt = Pcomt - Xt*Gammapcom;

xt = [ pt yt rt mt ut pcomt ];

Omega = (1/T)*(xt'*xt);

Omegahat = Omega;

% =========================================================================
% Maximize log likelihood from 2500 randomly-chosen starting points to
% ensure that the global optimium is achieved.
% =========================================================================

options = optimset('Display','off','LargeScale','off','MaxFunEvals',100000,'MaxIter',100000);

if model == 1
    m = 21;
else
    m = 17;
end

fstarmat = zeros(2500,1);
asigstarmat = zeros(m,2500);

for trial = 1:2500
    
    fprintf('estimating the SVAR: trial = %d\r',trial);
    
    asigstar = zeros(m,1);

    bveco = 200*rand(m,1) - 100*ones(m,1);

    [ bstar,cllfnstar ] = fminunc(@cllfn6,bveco,options);
    
    asigstar(m-5:m) = 1./abs(bstar(m-5:m));
    
    if model == 1
        
        asigstar(1) = bstar(1)*asigstar(m-4);
        asigstar(2:4) = bstar(2:4)*asigstar(m-3);
        asigstar(5:8) = bstar(5:8)*asigstar(m-2);
        asigstar(9:13) = bstar(9:13)*asigstar(m-1);
        asigstar(14:15) = bstar(14:15)*asigstar(m);
        
    else
        
        asigstar(1) = bstar(1)*asigstar(m-4);
        asigstar(2) = bstar(2)*asigstar(m-3);
        asigstar(3:4) = bstar(3:4)*asigstar(m-2);
        asigstar(5:6) = bstar(5:6)*asigstar(m-1);
        asigstar(7:11) = bstar(7:11)*asigstar(m);

    end
    
    asigstarmat(:,trial) = asigstar;
    
    fstarmat(trial) = cllfnstar;
    
end

% =========================================================================
% Select the best results.
% =========================================================================

[ fsort,isort ] = sort(fstarmat);
asigsort = asigstarmat(:,isort);

fstar = -fsort(1);

astar = asigsort(1:m-6,1);
sigstar = asigsort(m-5:m,1);

asigstar = [ astar ; sigstar ];

if model == 1
    
    Astar = [ 1 0 0 0 0 0 ; ...
              astar(1) 1 0 0 0 0 ; ...
              astar(2) astar(3) 1 0 0 astar(4) ; ...
              astar(5) astar(6) astar(7) 1 0 astar(8) ; ...
              astar(9) astar(10) astar(11) astar(12) 1 astar(13) ; ... 
              astar(14) astar(15) 0 0 0 1 ];

else
    
    Astar = [ 1 0 0 0 0 0 ; ...
              astar(1) 1 0 0 0 0 ; ...
              0 0 1 astar(2) 0 0 ; ...
              -1 astar(3) 0 1 astar(4) 0 ; ...
              -astar(6) 0 astar(5) astar(6) 1 0 ; ...
              astar(7) astar(8) astar(9) astar(10) astar(11) 1 ];
          
end

Sigmastar = diag(sigstar(1:6));

% =========================================================================
% Compute impulse responses.
% =========================================================================

[ ppimp,ypimp,rpimp,mpimp,upimp,cpimp ] = imps(Gamma,Astar,Sigmastar,1);
[ pyimp,yyimp,ryimp,myimp,uyimp,cyimp ] = imps(Gamma,Astar,Sigmastar,2);
[ primp,yrimp,rrimp,mrimp,urimp,crimp ] = imps(Gamma,Astar,Sigmastar,3);
[ pmimp,ymimp,rmimp,mmimp,umimp,cmimp ] = imps(Gamma,Astar,Sigmastar,4);
[ puimp,yuimp,ruimp,muimp,uuimp,cuimp ] = imps(Gamma,Astar,Sigmastar,5);
[ pcimp,ycimp,rcimp,mcimp,ucimp,ccimp ] = imps(Gamma,Astar,Sigmastar,6);

% =========================================================================
% Compute forecast error variance decompositions.
% =========================================================================

kvec = [ 2 4 8 12 16 20 ]';

[ ppdecomp,ypdecomp,rpdecomp,mpdecomp,updecomp,cpdecomp ] = vardecomp(Gamma,Astar,Sigmastar,kvec,1);
[ pydecomp,yydecomp,rydecomp,mydecomp,uydecomp,cydecomp ] = vardecomp(Gamma,Astar,Sigmastar,kvec,2);
[ prdecomp,yrdecomp,rrdecomp,mrdecomp,urdecomp,crdecomp ] = vardecomp(Gamma,Astar,Sigmastar,kvec,3);
[ pmdecomp,ymdecomp,rmdecomp,mmdecomp,umdecomp,cmdecomp ] = vardecomp(Gamma,Astar,Sigmastar,kvec,4);
[ pudecomp,yudecomp,rudecomp,mudecomp,uudecomp,cudecomp ] = vardecomp(Gamma,Astar,Sigmastar,kvec,5);
[ pcdecomp,ycdecomp,rcdecomp,mcdecomp,ucdecomp,ccdecomp ] = vardecomp(Gamma,Astar,Sigmastar,kvec,6);

% =========================================================================
% Compute standard errors for the intercept and slope coefficients using
% the more conservative OLS formula.
% =========================================================================

invXpX = eye(6*q+1)/XpX;

nuGammastar = [ nu(1:6) Gamma(1:6,:) ]';

nuGammastar = nuGammastar(:);

VnuGamma = kron((T/(T-6*q-1))*Omega,invXpX);

nuGammase = sqrt(diag(VnuGamma));

% =========================================================================
% Compute standard errors for the elements of A and Sigma using the
% formulas shown in Proposition 9.5 (p.373) of Helmut Lutkephol, New
% Introduction of Multiple Time Series, Spring-Verlag, 2006.
% =========================================================================

if model == 1
    
    Ra = zeros(36,15);
    
    Ra(2,1) = 1;
    Ra(3,2) = 1;
    Ra(4,5) = 1;
    Ra(5,9) = 1;
    Ra(6,14) = 1;
    Ra(9,3) = 1;
    Ra(10,6) = 1;
    Ra(11,10) = 1;
    Ra(12,15) = 1;
    Ra(16,7) = 1;
    Ra(17,11) = 1;
    Ra(23,12) = 1;
    Ra(33,4) = 1;
    Ra(34,8) = 1;
    Ra(35,13) = 1;
    
else

    Ra = zeros(36,11);
    
    Ra(2,1) = 1;
    Ra(5,6) = -1;
    Ra(6,7) = 1;
    Ra(10,3) = 1;
    Ra(12,8) = 1;
    Ra(17,5) = 1;
    Ra(18,9) = 1;
    Ra(21,2) = 1;
    Ra(23,6) = 1;
    Ra(24,10) = 1;
    Ra(28,4) = 1;
    Ra(30,11) = 1;

end

Rb = zeros(36,6);

Rb(1,1) = 1;
Rb(8,2) = 1;
Rb(15,3) = 1;
Rb(22,4) = 1;
Rb(29,5) = 1;
Rb(36,6) = 1;

K = zeros(36,36);

K(1,1) = 1;
K(2,7) = 1;
K(3,13) = 1;
K(4,19) = 1;
K(5,25) = 1;
K(6,31) = 1;
K(7,2) = 1;
K(8,8) = 1;
K(9,14) = 1;
K(10,20) = 1;
K(11,26) = 1;
K(12,32) = 1;
K(13,3) = 1;
K(14,9) = 1;
K(15,15) = 1;
K(16,21) = 1;
K(17,27) = 1;
K(18,33) = 1;
K(19,4) = 1;
K(20,10) = 1;
K(21,16) = 1;
K(22,22) = 1;
K(23,28) = 1;
K(24,34) = 1;
K(25,5) = 1;
K(26,11) = 1;
K(27,17) = 1;
K(28,23) = 1;
K(29,29) = 1;
K(30,35) = 1;
K(31,6) = 1;
K(32,12) = 1;
K(33,18) = 1;
K(34,24) = 1;
K(35,30) = 1;
K(36,36) = 1;

IvecAvecSiga = [ kron(Astar\Sigmastar,eye(6)/Sigmastar) ; ...
                 -kron(eye(6),eye(6)/Sigmastar) ];

IvecAvecSig = IvecAvecSiga*(eye(36)+K)*IvecAvecSiga';

dasig = [ Ra' zeros(m-6,36) ; zeros(6,36) Rb' ];

Iasig = dasig*IvecAvecSig*dasig';

Vasig = (1/T)*eye(m)/Iasig;

avecse = sqrt(diag(Vasig(1:m-6,1:m-6)));
sigmavecse = sqrt(diag(Vasig(m-5:m,m-5:m)));

if model == 1
    
    Astarse = [ 0 0 0 0 0 0 ; ...
                avecse(1) 0 0 0 0 0 ; ...
                avecse(2) avecse(3) 0 0 0 avecse(4) ; ...
                avecse(5) avecse(6) avecse(7) 0 0 avecse(8) ; ...
                avecse(9) avecse(10) avecse(11) avecse(12) 0 avecse(13) ; ... 
                avecse(14) avecse(15) 0 0 0 0 ];

else
    
    Astarse = [ 0 0 0 0 0 0 ; ...
                avecse(1) 0 0 0 0 0 ; ...
                0 0 0 avecse(2) 0 0 ; ...
                0 avecse(3) 0 0 avecse(4) 0 ; ...
                avecse(6) 0 avecse(5) avecse(6) 0 0 ; ...
                avecse(7) avecse(8) avecse(9) avecse(10) avecse(11) 0 ];
            
end

Sigmastarse = diag(sigmavecse(1:6));

% =========================================================================
% Compute standard errors for impulse responses and variance
% decompositions, treating each statistic as a function of the intercept
% and slope parameters and the coefficients of A and Sigma.
% =========================================================================

nuGamasigstar = [ nuGammastar ; asigstar ];

VnuGamasig = [ VnuGamma zeros(36*q+6,m) ; zeros(m,36*q+6) Vasig ];
    
dppimp = zeros(41,36*q+6+m);
dypimp = zeros(41,36*q+6+m);
drpimp = zeros(41,36*q+6+m);
dmpimp = zeros(41,36*q+6+m);
dupimp = zeros(41,36*q+6+m);
dcpimp = zeros(41,36*q+6+m);

dpyimp = zeros(41,36*q+6+m);
dyyimp = zeros(41,36*q+6+m);
dryimp = zeros(41,36*q+6+m);
dmyimp = zeros(41,36*q+6+m);
duyimp = zeros(41,36*q+6+m);
dcyimp = zeros(41,36*q+6+m);

dprimp = zeros(41,36*q+6+m);
dyrimp = zeros(41,36*q+6+m);
drrimp = zeros(41,36*q+6+m);
dmrimp = zeros(41,36*q+6+m);
durimp = zeros(41,36*q+6+m);
dcrimp = zeros(41,36*q+6+m);

dpmimp = zeros(41,36*q+6+m);
dymimp = zeros(41,36*q+6+m);
drmimp = zeros(41,36*q+6+m);
dmmimp = zeros(41,36*q+6+m);
dumimp = zeros(41,36*q+6+m);
dcmimp = zeros(41,36*q+6+m);

dpuimp = zeros(41,36*q+6+m);
dyuimp = zeros(41,36*q+6+m);
druimp = zeros(41,36*q+6+m);
dmuimp = zeros(41,36*q+6+m);
duuimp = zeros(41,36*q+6+m);
dcuimp = zeros(41,36*q+6+m);

dpcimp = zeros(41,36*q+6+m);
dycimp = zeros(41,36*q+6+m);
drcimp = zeros(41,36*q+6+m);
dmcimp = zeros(41,36*q+6+m);
ducimp = zeros(41,36*q+6+m);
dccimp = zeros(41,36*q+6+m);

dppdecomp = zeros(length(kvec),36*q+6+m);
dypdecomp = zeros(length(kvec),36*q+6+m);
drpdecomp = zeros(length(kvec),36*q+6+m);
dmpdecomp = zeros(length(kvec),36*q+6+m);
dupdecomp = zeros(length(kvec),36*q+6+m);
dcpdecomp = zeros(length(kvec),36*q+6+m);

dpydecomp = zeros(length(kvec),36*q+6+m);
dyydecomp = zeros(length(kvec),36*q+6+m);
drydecomp = zeros(length(kvec),36*q+6+m);
dmydecomp = zeros(length(kvec),36*q+6+m);
duydecomp = zeros(length(kvec),36*q+6+m);
dcydecomp = zeros(length(kvec),36*q+6+m);

dprdecomp = zeros(length(kvec),36*q+6+m);
dyrdecomp = zeros(length(kvec),36*q+6+m);
drrdecomp = zeros(length(kvec),36*q+6+m);
dmrdecomp = zeros(length(kvec),36*q+6+m);
durdecomp = zeros(length(kvec),36*q+6+m);
dcrdecomp = zeros(length(kvec),36*q+6+m);

dpmdecomp = zeros(length(kvec),36*q+6+m);
dymdecomp = zeros(length(kvec),36*q+6+m);
drmdecomp = zeros(length(kvec),36*q+6+m);
dmmdecomp = zeros(length(kvec),36*q+6+m);
dumdecomp = zeros(length(kvec),36*q+6+m);
dcmdecomp = zeros(length(kvec),36*q+6+m);

dpudecomp = zeros(length(kvec),36*q+6+m);
dyudecomp = zeros(length(kvec),36*q+6+m);
drudecomp = zeros(length(kvec),36*q+6+m);
dmudecomp = zeros(length(kvec),36*q+6+m);
duudecomp = zeros(length(kvec),36*q+6+m);
dcudecomp = zeros(length(kvec),36*q+6+m);

dpcdecomp = zeros(length(kvec),36*q+6+m);
dycdecomp = zeros(length(kvec),36*q+6+m);
drcdecomp = zeros(length(kvec),36*q+6+m);
dmcdecomp = zeros(length(kvec),36*q+6+m);
ducdecomp = zeros(length(kvec),36*q+6+m);
dccdecomp = zeros(length(kvec),36*q+6+m);

for i = 1:36*q+6+m
    
    epsvec = zeros(36*q+6+m,1);
    epsvec(i) = 0.0001;
    
    nuGamasigvec = nuGamasigstar + epsvec;
    
    nuGammamat = reshape(nuGamasigvec(1:36*q+6),6*q+1,6)';
    
    Gammamat = [ nuGammamat(:,2:6*q+1) ; eye(6*(q-1)) zeros(6*(q-1),6) ];
    
    asigvec = nuGamasigvec(36*q+6+1:36*q+6+m);
    
    avec = asigvec(1:m-6);
    sigvec = asigvec(m-5:m);
   
    if model == 1
    
        Amat = [ 1 0 0 0 0 0 ; ...
                avec(1) 1 0 0 0 0 ; ...
                avec(2) avec(3) 1 0 0 avec(4) ; ...
                avec(5) avec(6) avec(7) 1 0 avec(8) ; ...
                avec(9) avec(10) avec(11) avec(12) 1 avec(13) ; ... 
                avec(14) avec(15) 0 0 0 1 ];

    else
    
        Amat = [ 1 0 0 0 0 0 ; ...
                avec(1) 1 0 0 0 0 ; ...
                0 0 1 avec(2) 0 0 ; ...
                -1 avec(3) 0 1 avec(4) 0 ; ...
                -avec(6) 0 avec(5) avec(6) 1 0 ; ...
                avec(7) avec(8) avec(9) avec(10) avec(11) 1 ];
            
    end

    Sigmamat = diag(sigvec(1:6));
    
    [ ppimpvec,ypimpvec,rpimpvec,mpimpvec,upimpvec,cpimpvec ] = imps(Gammamat,Amat,Sigmamat,1);
    [ pyimpvec,yyimpvec,ryimpvec,myimpvec,uyimpvec,cyimpvec ] = imps(Gammamat,Amat,Sigmamat,2);
    [ primpvec,yrimpvec,rrimpvec,mrimpvec,urimpvec,crimpvec ] = imps(Gammamat,Amat,Sigmamat,3);
    [ pmimpvec,ymimpvec,rmimpvec,mmimpvec,umimpvec,cmimpvec ] = imps(Gammamat,Amat,Sigmamat,4);
    [ puimpvec,yuimpvec,ruimpvec,muimpvec,uuimpvec,cuimpvec ] = imps(Gammamat,Amat,Sigmamat,5);
    [ pcimpvec,ycimpvec,rcimpvec,mcimpvec,ucimpvec,ccimpvec ] = imps(Gammamat,Amat,Sigmamat,6);
    
    dppimp(:,i) = (ppimpvec-ppimp)/epsvec(i);
    dypimp(:,i) = (ypimpvec-ypimp)/epsvec(i);
    drpimp(:,i) = (rpimpvec-rpimp)/epsvec(i);
    dmpimp(:,i) = (mpimpvec-mpimp)/epsvec(i);
    dupimp(:,i) = (upimpvec-upimp)/epsvec(i);
    dcpimp(:,i) = (cpimpvec-cpimp)/epsvec(i);
    
    dpyimp(:,i) = (pyimpvec-pyimp)/epsvec(i);
    dyyimp(:,i) = (yyimpvec-yyimp)/epsvec(i);
    dryimp(:,i) = (ryimpvec-ryimp)/epsvec(i);
    dmyimp(:,i) = (myimpvec-myimp)/epsvec(i);
    duyimp(:,i) = (uyimpvec-uyimp)/epsvec(i);
    dcyimp(:,i) = (cyimpvec-cyimp)/epsvec(i);
    
    dprimp(:,i) = (primpvec-primp)/epsvec(i);
    dyrimp(:,i) = (yrimpvec-yrimp)/epsvec(i);
    drrimp(:,i) = (rrimpvec-rrimp)/epsvec(i);
    dmrimp(:,i) = (mrimpvec-mrimp)/epsvec(i);
    durimp(:,i) = (urimpvec-urimp)/epsvec(i);
    dcrimp(:,i) = (crimpvec-crimp)/epsvec(i);

    dpmimp(:,i) = (pmimpvec-pmimp)/epsvec(i);
    dymimp(:,i) = (ymimpvec-ymimp)/epsvec(i);
    drmimp(:,i) = (rmimpvec-rmimp)/epsvec(i);
    dmmimp(:,i) = (mmimpvec-mmimp)/epsvec(i);
    dumimp(:,i) = (umimpvec-umimp)/epsvec(i);
    dcmimp(:,i) = (cmimpvec-cmimp)/epsvec(i);

    dpuimp(:,i) = (puimpvec-puimp)/epsvec(i);
    dyuimp(:,i) = (yuimpvec-yuimp)/epsvec(i);
    druimp(:,i) = (ruimpvec-ruimp)/epsvec(i);
    dmuimp(:,i) = (muimpvec-muimp)/epsvec(i);
    duuimp(:,i) = (uuimpvec-uuimp)/epsvec(i);
    dcuimp(:,i) = (cuimpvec-cuimp)/epsvec(i);

    dpcimp(:,i) = (pcimpvec-pcimp)/epsvec(i);
    dycimp(:,i) = (ycimpvec-ycimp)/epsvec(i);
    drcimp(:,i) = (rcimpvec-rcimp)/epsvec(i);
    dmcimp(:,i) = (mcimpvec-mcimp)/epsvec(i);
    ducimp(:,i) = (ucimpvec-ucimp)/epsvec(i);
    dccimp(:,i) = (ccimpvec-ccimp)/epsvec(i);
    
    [ ppdecompvec,ypdecompvec,rpdecompvec,mpdecompvec,updecompvec,cpdecompvec ] = vardecomp(Gammamat,Amat,Sigmamat,kvec,1);
    [ pydecompvec,yydecompvec,rydecompvec,mydecompvec,uydecompvec,cydecompvec ] = vardecomp(Gammamat,Amat,Sigmamat,kvec,2);
    [ prdecompvec,yrdecompvec,rrdecompvec,mrdecompvec,urdecompvec,crdecompvec ] = vardecomp(Gammamat,Amat,Sigmamat,kvec,3);
    [ pmdecompvec,ymdecompvec,rmdecompvec,mmdecompvec,umdecompvec,cmdecompvec ] = vardecomp(Gammamat,Amat,Sigmamat,kvec,4);
    [ pudecompvec,yudecompvec,rudecompvec,mudecompvec,uudecompvec,cudecompvec ] = vardecomp(Gammamat,Amat,Sigmamat,kvec,5);
    [ pcdecompvec,ycdecompvec,rcdecompvec,mcdecompvec,ucdecompvec,ccdecompvec ] = vardecomp(Gammamat,Amat,Sigmamat,kvec,6);
    
    dppdecomp(:,i) = (ppdecompvec-ppdecomp)/epsvec(i);
    dypdecomp(:,i) = (ypdecompvec-ypdecomp)/epsvec(i);
    drpdecomp(:,i) = (rpdecompvec-rpdecomp)/epsvec(i);
    dmpdecomp(:,i) = (mpdecompvec-mpdecomp)/epsvec(i);
    dupdecomp(:,i) = (updecompvec-updecomp)/epsvec(i);
    dcpdecomp(:,i) = (cpdecompvec-cpdecomp)/epsvec(i);
    
    dpydecomp(:,i) = (pydecompvec-pydecomp)/epsvec(i);
    dyydecomp(:,i) = (yydecompvec-yydecomp)/epsvec(i);
    drydecomp(:,i) = (rydecompvec-rydecomp)/epsvec(i);
    dmydecomp(:,i) = (mydecompvec-mydecomp)/epsvec(i);
    duydecomp(:,i) = (uydecompvec-uydecomp)/epsvec(i);
    dcydecomp(:,i) = (cydecompvec-cydecomp)/epsvec(i);
    
    dprdecomp(:,i) = (prdecompvec-prdecomp)/epsvec(i);
    dyrdecomp(:,i) = (yrdecompvec-yrdecomp)/epsvec(i);
    drrdecomp(:,i) = (rrdecompvec-rrdecomp)/epsvec(i);
    dmrdecomp(:,i) = (mrdecompvec-mrdecomp)/epsvec(i);
    durdecomp(:,i) = (urdecompvec-urdecomp)/epsvec(i);
    dcrdecomp(:,i) = (crdecompvec-crdecomp)/epsvec(i);

    dpmdecomp(:,i) = (pmdecompvec-pmdecomp)/epsvec(i);
    dymdecomp(:,i) = (ymdecompvec-ymdecomp)/epsvec(i);
    drmdecomp(:,i) = (rmdecompvec-rmdecomp)/epsvec(i);
    dmmdecomp(:,i) = (mmdecompvec-mmdecomp)/epsvec(i);
    dumdecomp(:,i) = (umdecompvec-umdecomp)/epsvec(i);
    dcmdecomp(:,i) = (cmdecompvec-cmdecomp)/epsvec(i);

    dpudecomp(:,i) = (pudecompvec-pudecomp)/epsvec(i);
    dyudecomp(:,i) = (yudecompvec-yudecomp)/epsvec(i);
    drudecomp(:,i) = (rudecompvec-rudecomp)/epsvec(i);
    dmudecomp(:,i) = (mudecompvec-mudecomp)/epsvec(i);
    duudecomp(:,i) = (uudecompvec-uudecomp)/epsvec(i);
    dcudecomp(:,i) = (cudecompvec-cudecomp)/epsvec(i);

    dpcdecomp(:,i) = (pcdecompvec-pcdecomp)/epsvec(i);
    dycdecomp(:,i) = (ycdecompvec-ycdecomp)/epsvec(i);
    drcdecomp(:,i) = (rcdecompvec-rcdecomp)/epsvec(i);
    dmcdecomp(:,i) = (mcdecompvec-mcdecomp)/epsvec(i);
    ducdecomp(:,i) = (ucdecompvec-ucdecomp)/epsvec(i);
    dccdecomp(:,i) = (ccdecompvec-ccdecomp)/epsvec(i);

end

ppimpse = sqrt(diag(dppimp*VnuGamasig*dppimp'));
ypimpse = sqrt(diag(dypimp*VnuGamasig*dypimp'));
rpimpse = sqrt(diag(drpimp*VnuGamasig*drpimp'));
mpimpse = sqrt(diag(dmpimp*VnuGamasig*dmpimp'));
upimpse = sqrt(diag(dupimp*VnuGamasig*dupimp'));
cpimpse = sqrt(diag(dcpimp*VnuGamasig*dcpimp'));

pyimpse = sqrt(diag(dpyimp*VnuGamasig*dpyimp'));
yyimpse = sqrt(diag(dyyimp*VnuGamasig*dyyimp'));
ryimpse = sqrt(diag(dryimp*VnuGamasig*dryimp'));
myimpse = sqrt(diag(dmyimp*VnuGamasig*dmyimp'));
uyimpse = sqrt(diag(duyimp*VnuGamasig*duyimp'));
cyimpse = sqrt(diag(dcyimp*VnuGamasig*dcyimp'));

primpse = sqrt(diag(dprimp*VnuGamasig*dprimp'));
yrimpse = sqrt(diag(dyrimp*VnuGamasig*dyrimp'));
rrimpse = sqrt(diag(drrimp*VnuGamasig*drrimp'));
mrimpse = sqrt(diag(dmrimp*VnuGamasig*dmrimp'));
urimpse = sqrt(diag(durimp*VnuGamasig*durimp'));
crimpse = sqrt(diag(dcrimp*VnuGamasig*dcrimp'));

pmimpse = sqrt(diag(dpmimp*VnuGamasig*dpmimp'));
ymimpse = sqrt(diag(dymimp*VnuGamasig*dymimp'));
rmimpse = sqrt(diag(drmimp*VnuGamasig*drmimp'));
mmimpse = sqrt(diag(dmmimp*VnuGamasig*dmmimp'));
umimpse = sqrt(diag(dumimp*VnuGamasig*dumimp'));
cmimpse = sqrt(diag(dcmimp*VnuGamasig*dcmimp'));

puimpse = sqrt(diag(dpuimp*VnuGamasig*dpuimp'));
yuimpse = sqrt(diag(dyuimp*VnuGamasig*dyuimp'));
ruimpse = sqrt(diag(druimp*VnuGamasig*druimp'));
muimpse = sqrt(diag(dmuimp*VnuGamasig*dmuimp'));
uuimpse = sqrt(diag(duuimp*VnuGamasig*duuimp'));
cuimpse = sqrt(diag(dcuimp*VnuGamasig*dcuimp'));

pcimpse = sqrt(diag(dpcimp*VnuGamasig*dpcimp'));
ycimpse = sqrt(diag(dycimp*VnuGamasig*dycimp'));
rcimpse = sqrt(diag(drcimp*VnuGamasig*drcimp'));
mcimpse = sqrt(diag(dmcimp*VnuGamasig*dmcimp'));
ucimpse = sqrt(diag(ducimp*VnuGamasig*ducimp'));
ccimpse = sqrt(diag(dccimp*VnuGamasig*dccimp'));

ppdecompse = sqrt(diag(dppdecomp*VnuGamasig*dppdecomp'));
ypdecompse = sqrt(diag(dypdecomp*VnuGamasig*dypdecomp'));
rpdecompse = sqrt(diag(drpdecomp*VnuGamasig*drpdecomp'));
mpdecompse = sqrt(diag(dmpdecomp*VnuGamasig*dmpdecomp'));
updecompse = sqrt(diag(dupdecomp*VnuGamasig*dupdecomp'));
cpdecompse = sqrt(diag(dcpdecomp*VnuGamasig*dcpdecomp'));

pydecompse = sqrt(diag(dpydecomp*VnuGamasig*dpydecomp'));
yydecompse = sqrt(diag(dyydecomp*VnuGamasig*dyydecomp'));
rydecompse = sqrt(diag(drydecomp*VnuGamasig*drydecomp'));
mydecompse = sqrt(diag(dmydecomp*VnuGamasig*dmydecomp'));
uydecompse = sqrt(diag(duydecomp*VnuGamasig*duydecomp'));
cydecompse = sqrt(diag(dcydecomp*VnuGamasig*dcydecomp'));

prdecompse = sqrt(diag(dprdecomp*VnuGamasig*dprdecomp'));
yrdecompse = sqrt(diag(dyrdecomp*VnuGamasig*dyrdecomp'));
rrdecompse = sqrt(diag(drrdecomp*VnuGamasig*drrdecomp'));
mrdecompse = sqrt(diag(dmrdecomp*VnuGamasig*dmrdecomp'));
urdecompse = sqrt(diag(durdecomp*VnuGamasig*durdecomp'));
crdecompse = sqrt(diag(dcrdecomp*VnuGamasig*dcrdecomp'));

pmdecompse = sqrt(diag(dpmdecomp*VnuGamasig*dpmdecomp'));
ymdecompse = sqrt(diag(dymdecomp*VnuGamasig*dymdecomp'));
rmdecompse = sqrt(diag(drmdecomp*VnuGamasig*drmdecomp'));
mmdecompse = sqrt(diag(dmmdecomp*VnuGamasig*dmmdecomp'));
umdecompse = sqrt(diag(dumdecomp*VnuGamasig*dumdecomp'));
cmdecompse = sqrt(diag(dcmdecomp*VnuGamasig*dcmdecomp'));

pudecompse = sqrt(diag(dpudecomp*VnuGamasig*dpudecomp'));
yudecompse = sqrt(diag(dyudecomp*VnuGamasig*dyudecomp'));
rudecompse = sqrt(diag(drudecomp*VnuGamasig*drudecomp'));
mudecompse = sqrt(diag(dmudecomp*VnuGamasig*dmudecomp'));
uudecompse = sqrt(diag(duudecomp*VnuGamasig*duudecomp'));
cudecompse = sqrt(diag(dcudecomp*VnuGamasig*dcudecomp'));

pcdecompse = sqrt(diag(dpcdecomp*VnuGamasig*dpcdecomp'));
ycdecompse = sqrt(diag(dycdecomp*VnuGamasig*dycdecomp'));
rcdecompse = sqrt(diag(drcdecomp*VnuGamasig*drcdecomp'));
mcdecompse = sqrt(diag(dmcdecomp*VnuGamasig*dmcdecomp'));
ucdecompse = sqrt(diag(ducdecomp*VnuGamasig*ducdecomp'));
ccdecompse = sqrt(diag(dccdecomp*VnuGamasig*dccdecomp'));

% =========================================================================
% Report results.
% =========================================================================

fprintf('Maximized log likelihood = \r\r');
disp(fstar);
fprintf('MLE of A = \r\r')
disp(Astar);
fprintf('MLE of Sigma = \r\r')
disp(Sigmastar);
fprintf('Percentages of k-quarter ahead variances in output and prices \r');
fprintf('due to monetary policy shocks: \r\r');
disp([kvec yrdecomp prdecomp]);
fprintf('Percentages of k-quarter ahead variances in output and prices \r');
fprintf('due to money demand shocks: \r\r');
disp([kvec ymdecomp pmdecomp]);
fprintf('Percentages of k-quarter ahead variances in output and prices \r');
fprintf('due to monetary system shocks: \r\r');
disp([kvec yudecomp pudecomp]);

plot([ rrimp mrimp yrimp primp ]);

fprintf('Standard errors for A = \r\r')
disp(Astarse);
fprintf('Standard errors for Sigma = \r\r')
disp(Sigmastarse);
fprintf('Standard errors for variances in output and \r');
fprintf('prices due to monetary policy shocks: \r\r');
disp([kvec yrdecompse prdecompse]);
fprintf('Standard errors for variances in output and \r');
fprintf('prices due to money demand shocks: \r\r');
disp([kvec ymdecompse pmdecompse]);
fprintf('Standard errors for variances in output and \r');
fprintf('prices due to monetary system shocks: \r\r');
disp([kvec yudecompse pudecompse]);

fprintf('Standard errors for impulse responses are stored in XYimpse \r');
fprintf('and for variance decompositions in XYdecompse \r');
fprintf('where X, Y = {p,y,r,m,u,c} \r\r');

fprintf('OLS intercept and slope coefficients are stored in nuGammastar \r');
fprintf('and their standard errors in nuGammase \r')
